#!/bin/sh

############################################################################
#
# MODULE:       v.geobed 0.1 (alpha)
# AUTHOR(S):    Ivan Marchesini <ivan.marchesini AT irpi.cnr.it>
# PURPOSE:      Calculates geologic bedding attitudes starting from bedding traces and DEM
# COPYRIGHT:    (C) 2011 Ivan Marchesini
#
#               This program is free software under the GNU General Public
#               License (>=v2). Read the file COPYING that comes with GRASS
#               for details.
#
#############################################################################



#attenzione in input devono essere date le tracce dei markers (m) geologici 3d (anche se forse non serve che siano 3d) ed i punti 3d da queste estratti

START=$(date +%s)

bed_trc=$1
dtm=$2
sinuos=$3
output=$4
contour=$5

#faccio partire il monitor
d.mon start=x0

#aggiungo il valore di sinuosità bidimensionale
echo "I select only sinuous lines ........"
v.db.addcol map=$bed_trc layer=1 'columns=sinuos double precision' 
v.to.db map=$bed_trc type=line layer=1 qlayer=1 option=sinuous units=meters columns=sinuos 
eval `v.info -t map=$bed_trc layer=1`
linee_iniziali=$lines

#drappeggio le linee sul DEM
echo "I drape the beddings on the dtm ........."
eval `r.info -s map=$dtm`
region_inc=`echo "$nsres*2" | bc -l | cut -f1 -d'.'`
g.region vect=$bed_trc res=$nsres e=e+$region_inc w=w-$region_inc s=s-$region_inc n=n+$region_inc
v.drape input=$bed_trc type=line rast=$dtm output=bedd_3d method=nearest scale=1.0 layer=1 --o

#estraggo le sole linee che hanno sinuosità sopra il valore fornito o che hanno sinuosità nulla (linee chiuse)
v.extract input=bedd_3d output=bed_trc_3d_sinuos type=line layer=1 where="sinuos>$sinuos or sinuos is null" new=-1 --o
eval `v.info -t map=bed_trc_3d_sinuos layer=1`
echo "
-------------------------------------------------------------------
----------- SELEZIONATE $lines linee di $linee_iniziali iniziali --
-------------------------------------------------------------------
" 
sleep 3

#setto la variabile risoluzione di lavoro alla risoluzione del dem
res_dem=$nsres


#inizializzo un contatore
k=1
for i in `v.category -g input=bed_trc_3d_sinuos option=print type=line`
	do

	
		#estraggo la prima linea 
		v.extract input=bedd_3d output=m_bacino_$i list=$i --o	
		
		#calcolo le coordinate iniziali e finali di questa linea
		g.region vect=m_bacino_$i res=$res_dem -a e=e+20 w=w-20 s=s-20 n=n+20
		startx=`v.to.db -p map=m_bacino_$i type=line layer=1 qlayer=1 option=start units=meters | sed 1d | cut -f2 -d'|'`
		starty=`v.to.db -p map=m_bacino_$i type=line layer=1 qlayer=1 option=start units=meters | sed 1d | cut -f3 -d'|'`
		startz=`v.to.db -p map=m_bacino_$i type=line layer=1 qlayer=1 option=start units=meters | sed 1d | cut -f4 -d'|'`
		endx=`v.to.db -p map=m_bacino_$i type=line layer=1 qlayer=1 option=end units=meters | sed 1d | cut -f2 -d'|'`
		endy=`v.to.db -p map=m_bacino_$i type=line layer=1 qlayer=1 option=end units=meters | sed 1d | cut -f3 -d'|'`
		endz=`v.to.db -p map=m_bacino_$i type=line layer=1 qlayer=1 option=end units=meters | sed 1d | cut -f4 -d'|'`

		
		#creo un area chiusa costituita dal marker più la linea che unisce gli estremi
		echo "L 2 1
		$startx $starty $startz
		$endx $endy $endz
		1 $i" | v.edit -n tool=add map=m_bacino_$i snap=node thresh=1
		v.to.points -v -i input=m_bacino_$i output=m_bacino_pnt_$i type=line llayer=1 dmax=$res_dem --o

		
		#calcolo "l'ampiezza media" dell'area definita dal marker più la linea che unisce gli estremi
		echo "L 2 1
		$startx $starty $startz
		$endx $endy $endz
		1 $i" | v.in.ascii -nz out=connect format=standard --overwrite
		v.type input=m_bacino_$i output=m_bacino_bound_$i type=line,boundary --o
		v.to.3d -r input=m_bacino_bound_$i output=m_bacino_bound_2d_$i type=boundary layer=1 --o
		v.clean input=m_bacino_bound_2d_$i output=m_bacino_bound_2d_break_$i type=boundary tool=break --overwrite 
		v.centroids input=m_bacino_bound_2d_break_$i output=m_bacino_bound_cent_$i option=add layer=1 cat=1 step=1 --o 
		area=`v.to.db -p -c map=m_bacino_bound_cent_$i type=centroid layer=1 qlayer=1 option=area units=meters columns=m | cut -f2 -d':' | tr -d ' ' | cut -f1 -d'.'`
		base=`v.to.db -p map=connect type=line layer=1 qlayer=1 option=length units=meters columns=p | sed 1d | cut -f2 -d'|' | cut -f1 -d'.'`
		heigh=`echo "$area/($base+1)" | bc -l | cut -f1 -d'.'`

		
		#verifico se l'altezza media è molto bassa (cioè ho un marker molto allungato) o se l'altezza media è molto alta (cioè ho un marker che si piega su se stesso in modo molto stretto) 
		#e nel caso cambio la risoluzione in modo tale che il calcolo del piano inclinato avvenga anche per linee molto diritte
		heigh_min=`echo "3*$res_dem" | bc -l`
		heigh_max=`echo "$area/(3*$res_dem)" | bc -l | cut -f1 -d'.'`
		if [ $heigh -lt $heigh_min ]
		then
			res=`echo "$heigh/5" | bc -l`
			g.region res=$res
			echo "la risoluzione è $res"
		elif [ $heigh -gt $heigh_max ] && [$heigh -ne $area ]
		then
			res=`echo "$base/5" | bc -l | cut -f1 -d'.'`
			g.region res=$res
			echo "la risoluzione è $res"
		else
			echo "la risoluzione è $res_dem"
		fi
		
		#Produco la mappa del "piano" passante per il marker e le sue mappe derivate di slope ed aspect
		v.extract input=m_bacino_pnt_$i output=m_pt_3dim_$i layer=2 where="lcat=$i" --o
		v.to.rast input=m_pt_3dim_$i output=m_pt_3dim_$i use=z type=point layer=2 value=1 rows=4096 --overwrite
		#read
		NOW=$(date +%s)
		DIFF=$(( $NOW - $START ))
		echo "
		-------------------------------------------------------------------
		--------- Sto analizzando la linea $k di $lines linee totali ------
		--------- Sto lavorando da $DIFF secondi --------------------------
		--------- Categoria della linea: $i -------------------------------
		-------------------------------------------------------------------
		" 
		r.surf.nnbathy input=m_pt_3dim_$i output=m_pt_3dim_nn alg=l --overwrite 
		v.to.rast input=m_bacino_bound_cent_$i output=MASK use=val type=area layer=1 value=1 rows=4096 --o
		r.mapcalc "m_pt_3dim_nn_$i=m_pt_3dim_nn"
		g.remove rast=MASK
		d.erase
		d.rast m_pt_3dim_nn_$i -o
		d.vect $contour color=red 
		d.vect m_bacino_bound_cent_$i fcolor=none width=2 
		r.slope.aspect elevation=m_pt_3dim_nn_$i slope=slope_$i aspect=aspect_$i format=degrees prec=float zfactor=1.0 min_slp_allowed=0.0 --overwrite 
		r.mapcalc "m_pt_3dim_1_nn_poly=m_pt_3dim_nn_$i/m_pt_3dim_nn_$i"
		r.to.vect input=m_pt_3dim_1_nn_poly output=m_pt_3dim_1_nn_poly_$i feature=area --overwrite 
		thresh=`echo "$res_dem*$res_dem*2" | bc -l`
		v.clean input=m_pt_3dim_1_nn_poly_$i output=m_pt_3dim_1_nn_poly_clean_$i type=area tool=rmarea thresh=$thresh --o
		
		#assegno le variabili slope ed azimuth al poligono che rappresenta il marker
		g.copy rast=m_pt_3dim_1_nn_poly,MASK
			#assegno slope
		eval `r.univar -eg map=slope_$i`	
		slope50=$median		
		echo "
		-------------------------------------------------------------------
		--------- Sto analizzando la linea $k di $lines linee totali ------
		--------- Sto lavorando da $DIFF secondi --------------------------
		--------- Categoria della linea: $i -------------------------------
		-------------------------------------------------------------------
		" 
		g.remove rast=MASK
		v.db.addcol map=m_pt_3dim_1_nn_poly_clean_$i columns="slopestdev double precision, slope50 double precision, invslope50 double precision, aspect double precision, azimuth double precision, circvar double precision, angstddev double precision"
		slopestdev=`echo $stddev | cut -f1 -d'.'`
		slope50=`echo $slope50 | cut -f1 -d'.'`
		inv_slope50=`echo "90-$slope50" | bc -l`		
		v.db.update map=m_pt_3dim_1_nn_poly_clean_$i layer=1 column=slopestdev value=$slopestdev
		v.db.update map=m_pt_3dim_1_nn_poly_clean_$i layer=1 column=slope50 value=$slope50
		v.db.update map=m_pt_3dim_1_nn_poly_clean_$i layer=1 column=invslope50 value=$inv_slope50
		#read
			#assegno azimuth
		r.mapcalc "sen_aspect_$i=sin(aspect_$i)"
		r.mapcalc "cos_aspect_$i=cos(aspect_$i)"
		eval `r.univar -eg map=sen_aspect_$i`
		mean_sen=$mean
		eval `r.univar -eg map=cos_aspect_$i`		
		mean_cos=$mean
		echo $mean_sen
		echo $mean_cos
		mean_sen_int=`echo "$mean_sen*1000000000" | bc -l | cut -f1 -d'.' `
		mean_cos_int=`echo "$mean_cos*1000000000" | bc -l | cut -f1 -d'.' `
		echo $mean_sen_int
		echo $mean_cos_int		
		if [ "$mean_sen_int" -gt "0" ] && [ "$mean_cos_int" -gt "0" ] 
		then
			aspect_avg=`echo "a($mean_sen/$mean_cos)*360/6.28" | bc -l`
			azimuth=`echo "90-$aspect_avg" | bc -l`
		elif [ "$mean_sen_int" -gt "0" ] && [ "$mean_cos_int" -lt "0" ] 
		then
			aspect_avg=`echo "a($mean_sen/$mean_cos)*360/6.28+180" | bc -l`
			azimuth=`echo "360-$aspect_avg+90" | bc -l`
		elif [ "$mean_sen_int" -lt "0" ] && [ "$mean_cos_int" -lt "0" ] 
		then
			aspect_avg=`echo "a($mean_sen/$mean_cos)*360/6.28+180" | bc -l`
			echo "l'aspect medio è: $aspect_avg"
			azimuth=`echo "360-$aspect_avg+90" | bc -l`
			echo "l'azimuth è: $azimuth"
		elif [ "$mean_sen_int" -lt "0" ] && [ "$mean_cos_int" -gt "0" ] 
		then
			aspect_avg=`echo "a($mean_sen/$mean_cos)*360/6.28+360" | bc -l`
			azimuth=`echo "-($aspect_avg-360)+90" | bc -l`
		fi
		azimuth=`echo $azimuth | cut -f1 -d'.'`
		#~ echo $aspect_avg
		#~ echo $azimuth
		v.db.update map=m_pt_3dim_1_nn_poly_clean_$i layer=1 column=aspect value=$aspect_avg
		v.db.update map=m_pt_3dim_1_nn_poly_clean_$i layer=1 column=azimuth value=$azimuth


			#assegno variabilità aspect
		eval `r.univar -g map=sen_aspect_$i`
		sen_sum=$sum
		eval `r.univar -g map=cos_aspect_$i`
		cos_sum=$sum
		n_cells=$n
		circvar=`echo "1-sqrt($sen_sum^2+$cos_sum^2)/$n" | bc -l`
		v.db.update map=m_pt_3dim_1_nn_poly_clean_$i layer=1 column=circvar value=$circvar
		
			#assegno variabilità aspect, altro metodo
		r.mapcalc "asp_semi_$i=if(aspect_$i<=180,aspect_$i,aspect_$i-360)"
		aspect_avg_int=`echo "$aspect_avg" | cut -f1 -d'.'`
		if [ $aspect_avg_int -le 180 ]
		then
			aspect_avg_new=$aspect_avg_int
		elif [ $aspect_avg_int -gt 180 ]
		then
			aspect_avg_new=`echo "$aspect_avg_int-360" | bc -l`
		fi
		r.mapcalc "diff_angoli_$i=asp_semi_$i-$aspect_avg_new"
		r.mapcalc "diff_angoli_semi_$i=if(abs(diff_angoli_$i)<=180,diff_angoli_$i,(360-abs(diff_angoli_$i)))"
		r.univar asp_semi_$i
		r.univar diff_angoli_semi_$i
		r.mapcalc "diff_angoli_semi_squared_$i=diff_angoli_semi_$i^2"
		r.univar diff_angoli_semi_squared_$i
		eval `r.univar -g diff_angoli_semi_squared_$i`
		angstddev=`echo "sqrt($sum/$n)" | bc -l`
		v.db.update map=m_pt_3dim_1_nn_poly_clean_$i layer=1 column=angstddev value=$angstddev

		
		#trasformo i centroidi delle aree dei markers in punti, li sposto al centro della region e poi, laddove ne esista più di uno, prendo solo il primo
		v.type input=m_pt_3dim_1_nn_poly_clean_$i output=ptbound_$i type=centroid,point  --o
		eval `g.region -c -g vect=m_pt_3dim_1_nn_poly_clean_$i`
		first_pt_cat=`v.category input=ptbound_$i option=print type=point layer=1 | tail -1`
		v.extract input=ptbound_$i output=pt_$i type=point list=$first_pt_cat --o		
		d.vect pt_$i
		point_cat=`v.to.db -p map="pt_$i" type=point layer=1 qlayer=1 option=cat columns=cat | sed 1d | sed 2,100d`
		point_easting=`v.to.db -p map=pt_$i type=point layer=1 qlayer=1 option=coor columns=x,y | cut -f2 -d'|' | sed 1d | sed 2,100d` 
		point_northing=`v.to.db -p map=pt_$i type=point layer=1 qlayer=1 option=coor columns=x,y | cut -f3 -d'|'  | sed 1d | sed 2,100d`
		x_move=`echo "$center_easting-$point_easting" | bc -l`
		y_move=`echo "$center_northing-$point_northing" | bc -l`
		v.edit tool=move map=pt_$i coords="$point_easting,$point_northing" move="$x_move,$y_move"
		d.vect pt_$i icon="geology/strike_circle" size=8
			
		#creo una mappa che contiene solo le giaciture medie per la zona di interesse
		if [ "$k" -eq "1" ]
		then
			eval `g.region vect=pt_$i -p -g`
			g.remove vect=giaciture_mediate
			g.copy vect=pt_$i,giaciture_mediate
			v.edit tool=catdel map=giaciture_mediate ids="1-99999" cats="1-99999" layer=1
			v.edit tool=catadd map=giaciture_mediate bbox="$w,$s,$e,$n" layer=1 cat=$i
			v.db.update map=giaciture_mediate layer=1 column=cat value=$i
			echo "creo la mappa delle giaciture mediate"
			#read
		else
			eval `g.region vect=pt_$i -p -g`
			v.patch -a -e input=pt_$i output=giaciture_mediate --o
			last_cat=`v.category input=giaciture_mediate option=print type=point layer=1 | tail -1`
			v.edit tool=catdel map=giaciture_mediate ids="1-99999" cats="$last_cat" layer=1
			v.edit tool=catadd map=giaciture_mediate bbox="$w,$s,$e,$n" layer=1 cat=$i
			v.db.update map=giaciture_mediate layer=1 column=cat value=$i where="cat=$last_cat"
			echo "patcho le mappe delle giaciture mediate"
		fi
		
		


		#rimuovo mappe temporanee
		g.remove vect=m_bacino_bound_$i
		g.remove vect=m_bacino_bound_2d_$i
		g.remove vect=m_bacino_bound_2d_break_$i
		g.remove vect=m_bacino_bound_cent_$i
		g.remove vect=m_bacino_pnt_$i
		g.remove vect=m_pt_3dim_1_nn_poly_$i
		g.remove vect=m_pt_3dim_1_nn_poly_clean_$i
		#g.rename vect=giac_temp_$i,giaciture		
		g.remove vect=m_pt_3d_delaunay_point_$i
		g.remove vect=m_pt_3d_delaunay_$i
		g.remove vect=m_bacino_$i
		g.remove vect=m_pt_3dim_$i
		g.remove vect=area_$i
		g.remove vect=ptbound_$i
		g.remove vect=pt_$i
		g.remove vect=pti_3d_$i
		g.remove vect=pti_3d_bis_$i
		g.remove vect=giac_temp_$i
		g.remove rast=cos_aspect_$i,sen_aspect_$i,m_pt_3dim_$i
		g.remove rast=aspect_$i
		g.remove rast=area_$i
		g.remove rast=slope_$i
		g.remove rast=m_pt_3dim_nn_$i

		#incremento il contatore
		k=`echo "$k+1" | bc -l`
		
	done
	#stranamente qgis non vede il layer della mappa giaciture_mediate, quindi creo un anuova mappa in output e gli cambio nome
	v.category input=giaciture_mediate output=$output option=chlayer type=point layer=1,1 cat=1 step=1 --o	
	g.region vect=bedd_3d
		
		

